<?php

/**
 * @file
 * All filters related functions and form.
 */

/**
 * Filters form.
 *
 * @param array $form
 *   : Form.
 * @param array $form_state
 *   : From state.
 * @param array $filters
 *   : Filters.
 * @param string $instanceSuffix
 *   : HAL Publications block instance number.
 *
 * @return array
 *   : Form.
 */
function _amu_hal_filters_form(array $form, array &$form_state, array $filters, $instanceSuffix) {
  $term = $_GET['term'];
  $author = explode('+', $_GET['author']);
  $year = explode('+', $_GET['year']);

  // Text filter.
  $form['term'] = [
    '#type' => 'textfield',
    '#title' => t('Mot(s) présent(s) dans le titre, résumé etc.'),
    '#default_value' => $term,
  ];

  // Authors filter.
  $form['author'] = [
    '#type' => 'select',
    '#id' => 'edit-author',
    '#multiple' => variable_get('multiple_authors' . $instanceSuffix)[1] == 1 ? TRUE : FALSE,
    '#title' => t('Auteur(s)'),
    '#default_value' => $author,
    '#options' => $filters['author'],
  ];

  // Years filter.
  $form['year'] = [
    '#type' => 'select',
    '#id' => 'edit-year',
    '#multiple' => variable_get('multiple_years' . $instanceSuffix)[1] == 1 ? TRUE : FALSE,
    '#title' => t('Année(s)'),
    '#default_value' => $year,
    '#options' => $filters['year'],
  ];

  $form['instanceSuffix'] = [
    '#type' => 'hidden',
    '#value' => $instanceSuffix,
  ];

  // Filter button.
  $form['submit'] = [
    '#type' => 'submit',
    '#value' => t('Rechercher'),
    '#submit' => [
      '_amu_hal_filters_submit',
    ],
  ];

  // Reset button is only created if filters have already been chosen.
  if ($_GET['term'] || $_GET['author'] || $_GET['year']) {
    // Reset button.
    $form['reset'] = [
      '#type' => 'submit',
      '#value' => t('Réinitialiser'),
      '#submit' => [
        '_amu_hal_filters_reset',
      ],
    ];
  }

  return $form;
}

/**
 * Resets all filters.
 *
 * @param array $form
 *   : Form.
 * @param array $form_state
 *   : From state.
 */
function _amu_hal_filters_reset(array $form, array &$form_state) {
  $url = explode("?", $form['#action'])[0];
  drupal_goto($url);
}

/**
 * Filters submit function: adds tje filters values to the url.
 *
 * @param array $form
 *   : Form.
 * @param array $form_state
 *   : From state.
 */
function _amu_hal_filters_submit(array $form, array &$form_state) {

  $url = explode("?", $form['#action'])[0];

  $instanceSuffix = $form_state['values']['instanceSuffix'];
  $multiple_authors = variable_get('multiple_authors' . $instanceSuffix)[1];
  $multiple_years = variable_get('multiple_years' . $instanceSuffix)[1];

  $options = [];

  if ($form_state['values']['term']) {
    $term = $form_state['values']['term'];
  }

  if ($form_state['values']['author']) {
    $authors = "";
    if ($multiple_authors == 0) {
      $authors = $form_state['values']['author'];
    }
    else {
      $last_author = end($form_state['values']['author']);
      foreach ($form_state['values']['author'] as $author) {
        if ($last_author == $author) {
          $authors .= $author;
        }
        else {
          $authors .= $author . "+";
        }
      }
    }
  }

  if ($form_state['values']['year']) {
    $years = "";
    if ($multiple_years == 0) {
      $years = $form_state['values']['year'];
    }
    else {
      $last_year = end($form_state['values']['year']);
      foreach ($form_state['values']['year'] as $year) {
        if ($last_year == $year) {
          $years .= $year;
        }
        else {
          $years .= $year . "+";
        }
      }
    }
  }

  if ($term) {
    $options["query"]["term"] = $term;
  }

  if ($authors) {
    $options["query"]["author"] = $authors;
  }

  if ($years) {
    $options["query"]["year"] = $years;
  }

  drupal_goto($url, $options);
}

/**
 * Function to get the filters values.
 *
 * @param string $instanceSuffix
 *   : HAL Publications block instance number.
 *
 * @return array
 *   : Filters values.
 */
function _amu_hal_get_filters($instanceSuffix) {
  $filters = [];
  $config = _amu_hal_get_configuration($instanceSuffix);

  // Authors IDs and full names from the users profiles.
  $filters['author'] = _amu_hal_ids($config->choices, TRUE, TRUE);
  // Years from archives ouvertes api.
  $filters['year'] = _amu_hal_api_facet_request($config->choices, 'producedDateY_i', TRUE);
  // None value for single select list authors.
  if (variable_get('multiple_authors' . $instanceSuffix)[1] == 0) {
    $filters['author'] = ["" => t("Auteur")] + $filters['author'];
  }
  // None value for single select list years.
  if (variable_get('multiple_years' . $instanceSuffix)[1] == 0) {
    $filters['year'] = ["" => t("Année")] + $filters['year'];
  }
  return $filters;
}

/**
 * Function to get any archives ouvertes api field options list for filters.
 *
 * @param array $choices
 *   : Team IDs.
 * @param string $facet
 *   : Facet id.
 * @param bool $reverse
 *   : True: results will be sorted in descendant order.
 *
 * @return array
 *   : Facets.
 */
function _amu_hal_api_facet_request(array $choices, $facet, $reverse = FALSE) {
  $clean_facets = [];
  // Build the request.
  $url = _amu_hal_facets_url($choices, $facet);
  // Send the request.
  $result = drupal_http_request($url);

  // Json_decode does not decode the request correctly,
  // Therefor these few lines of code are needed to fix the array.
  $unclean_facets = json_decode($result->data, TRUE)['facet_counts']['facet_fields'][$facet];
  foreach ($unclean_facets as $key => $value) {
    if ($key % 2 !== 0) {
      continue;
    }
    if ($unclean_facets[$key + 1] != 0) {
      $clean_facets[$value] = $value;
    }
  }

  if ($reverse) {
    arsort($clean_facets);
  }
  else {
    asort($clean_facets);
  }

  return $clean_facets;
}

/**
 * Creates request url for filters options list.
 *
 * @param array $choices
 *   : Team IDs.
 * @param string $field
 *   : Facet id.
 *
 * @return string
 *   : url string.
 */
function _amu_hal_facets_url(array $choices, $field) {
  $url = variable_get('amu_hal_url_ws') . 'search/' . variable_get('portal') . '/';
  $tabauthids = _amu_hal_ids($choices);
  $authidValues = '';
  if (count($tabauthids) > 0) {
    $authidValues = '(' . implode(' OR ', $tabauthids) . ')';
  }

  $url .= '?fq=authIdHal_s:' . urlencode($authidValues);
  $url .= '&rows=0&facet=true&facet.field=' . $field;

  return $url;
}
